<?php

namespace Restaurant\Googleapi\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Restaurant\Googleapi\Http\Requests\CampaignRequest;
use Restaurant\Googleapi\Interfaces\CampaignRepositoryInterface;
use Restaurant\Googleapi\Models\Campaign;

/**
 * Resource controller class for campaign.
 */
class CampaignResourceController extends BaseController
{

    /**
     * Initialize campaign resource controller.
     *
     * @param type CampaignRepositoryInterface $campaign
     *
     * @return null
     */
    public function __construct(CampaignRepositoryInterface $campaign)
    {
        parent::__construct();
        $this->repository = $campaign;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Restaurant\Googleapi\Repositories\Criteria\CampaignResourceCriteria::class);
    }

    /**
     * Display a list of campaign.
     *
     * @return Response
     */
    public function index(CampaignRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Restaurant\Googleapi\Repositories\Presenter\CampaignPresenter::class)
                ->$function();
        }

        $campaigns = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('googleapi::campaign.names'))
            ->view('googleapi::campaign.index', true)
            ->data(compact('campaigns', 'view'))
            ->output();
    }

    /**
     * Display campaign.
     *
     * @param Request $request
     * @param Model   $campaign
     *
     * @return Response
     */
    public function show(CampaignRequest $request, Campaign $campaign)
    {

        if ($campaign->exists) {
            $view = 'googleapi::campaign.show';
        } else {
            $view = 'googleapi::campaign.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('googleapi::campaign.name'))
            ->data(compact('campaign'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new campaign.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CampaignRequest $request)
    {

        $campaign = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('googleapi::campaign.name')) 
            ->view('googleapi::campaign.create', true) 
            ->data(compact('campaign'))
            ->output();
    }

    /**
     * Create new campaign.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CampaignRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $campaign                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('googleapi::campaign.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('googleapi/campaign/' . $campaign->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/googleapi/campaign'))
                ->redirect();
        }

    }

    /**
     * Show campaign for editing.
     *
     * @param Request $request
     * @param Model   $campaign
     *
     * @return Response
     */
    public function edit(CampaignRequest $request, Campaign $campaign)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('googleapi::campaign.name'))
            ->view('googleapi::campaign.edit', true)
            ->data(compact('campaign'))
            ->output();
    }

    /**
     * Update the campaign.
     *
     * @param Request $request
     * @param Model   $campaign
     *
     * @return Response
     */
    public function update(CampaignRequest $request, Campaign $campaign)
    {
        try {
            $attributes = $request->all();

            $campaign->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('googleapi::campaign.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('googleapi/campaign/' . $campaign->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('googleapi/campaign/' . $campaign->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the campaign.
     *
     * @param Model   $campaign
     *
     * @return Response
     */
    public function destroy(CampaignRequest $request, Campaign $campaign)
    {
        try {

            $campaign->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('googleapi::campaign.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('googleapi/campaign/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('googleapi/campaign/' . $campaign->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple campaign.
     *
     * @param Model   $campaign
     *
     * @return Response
     */
    public function delete(CampaignRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('googleapi::campaign.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('googleapi/campaign'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/googleapi/campaign'))
                ->redirect();
        }

    }

    /**
     * Restore deleted campaigns.
     *
     * @param Model   $campaign
     *
     * @return Response
     */
    public function restore(CampaignRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('googleapi::campaign.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/googleapi/campaign'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/googleapi/campaign/'))
                ->redirect();
        }

    }

}
